<?php
namespace App\Http\Controllers;

use App\Repositories\CmsContentTemplateRepository;
use App\Transformers\CmsContentTemplateTransformer;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use League\Fractal\Resource\Collection;
use League\Fractal\Manager;
use Illuminate\Validation\Rule;
use League\Fractal\Pagination\IlluminatePaginatorAdapter;
/**
 * Created by PhpStorm.
 * User: qinyaer
 * Date: 2019/4/29
 * Time: 下午15:45
 */

class CmsContentTemplateController extends BaseController
{
    public function __construct(CmsContentTemplateRepository $cmsContentTemplateRepository)
    {
        $this->cmsContentTemplate = $cmsContentTemplateRepository;
    }

    //模板列表
    public function index(Request $request)
    {
        $cmsTemplate = $this->cmsContentTemplate->index($request->all());

        $fractal = new Manager();
        $resource = new Collection($cmsTemplate, new CmsContentTemplateTransformer());
        $resource->setPaginator(new IlluminatePaginatorAdapter($cmsTemplate));
        $data = $fractal->createData($resource)->toArray();

        $data['extra'] = [
            'filters' => [
                'id',
                'city_id',
                'is_open',
                'status',
            ],
            'columns' => [
                'id',
                'city_id',
                'city_name',
                'title',
                'apply_type',
                'is_open',
                'status',
            ]
        ];
        return $data;
    }


    //新建模板
    public function create(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'title' => 'required|string',
            'apply_type' => ['required', Rule::in(0,1,2)],
        ]);
        if ($validator->fails()) {
            return $this->response->error($validator->errors()->first(), 500);
        }

        return  $this->cmsContentTemplate->create($request->all());

    }

    //修改模板状态
    public function edit(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'id' => 'required|exists:cms_content_template',
            'is_open' => ['required', Rule::in(0,1)],
            'apply_type' => ['required', Rule::in(0,1,2)],
            'city_id' => 'required',
        ]);
        if ($validator->fails()) {
            return $this->response->error($validator->errors()->first(), 500);
        }

        return  $this->cmsContentTemplate->edit($request->all());

    }

    /**
     * 修改模板名称
     * @param Request $request
     * @return mixed|void
     */
    public function editTemplateName(Request $request){
        $validator = Validator::make($request->all(), [
            'id' => 'required|exists:cms_content_template',
            'title' => 'required',
        ]);
        if ($validator->fails()) {
            return $this->response->error($validator->errors()->first(), 500);
        }
        return  $this->cmsContentTemplate->updateTemplateName($request->all());
    }

}