<?php
namespace App\Http\Controllers;

use App\Repositories\CmsContentTemplateSetRepository;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\Rule;
/**
 * Created by PhpStorm.
 * User: qinyaer
 * Date: 2019/4/29
 * Time: 下午18:37
 */

class CmsContentTemplateSetController extends BaseController
{
    public function __construct(CmsContentTemplateSetRepository $cmsContentTemplateSetRepository)
    {
        $this->cmsContentSetTemplate = $cmsContentTemplateSetRepository;
    }


    //点击配置
    public function set(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'tpl_id' => 'required',
        ]);
        if ($validator->fails()) {
            return $this->response->error($validator->errors()->first(), 500);
        }

        return  $this->cmsContentSetTemplate->set($request->all());

    }

    //banner设置
    public function bannerSet(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'tpl_id' => 'required|integer',
            'area_type' => ['required', Rule::in(0,1,2,3,4,5)],
        ]);
        if ($validator->fails()) {
            return $this->response->error($validator->errors()->first(), 500);
        }

        return  $this->cmsContentSetTemplate->bannerSet($request->all());

    }

    //专题广告设置
    public function advertisement(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'rule' => 'required',
            'tpl_id' => 'required|integer',
            'area_type' => ['required', Rule::in(0,1,2,3,4,5)],
        ]);
        if ($validator->fails()) {
            return $this->response->error($validator->errors()->first(), 500);
        }

        return  $this->cmsContentSetTemplate->advertisementSet($request->all());

    }

    //商品楼层设置
    public function floorSet(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'rule' => 'required',
            'tpl_id' => 'required|integer',
            'area_type' => ['required', Rule::in(0,1,2,3,4,5)],
        ]);
        if ($validator->fails()) {
            return $this->response->error($validator->errors()->first(), 500);
        }

        return  $this->cmsContentSetTemplate->floorSet($request->all());

    }

    //分类专题设置
    public function categorySet(Request $request)
    {
//        Log::debug('request-param-categorySet:'.json_encode($request->all()));
        $validator = Validator::make($request->all(), [
            'rule' => 'required',
            'tpl_id' => 'required|integer',
            'area_type' => ['required', Rule::in(0,1,2,3,4,5)],
        ]);
        if ($validator->fails()) {
            return $this->response->error($validator->errors()->first(), 500);
        }
        return  $this->cmsContentSetTemplate->categorySet($request->all());

    }

    //左一右二设置
    public function subjectOne(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'rule' => 'required|array',
            'tpl_id' => 'required|integer',
            'area_type' => ['required', Rule::in(0,1,2,3,4,5)],
        ]);
        if ($validator->fails()) {
            return $this->response->error($validator->errors()->first(), 500);
        }

        return  $this->cmsContentSetTemplate->subjectOne($request->all());

    }

    //上一下三配置
    public function subjectTwo(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'rule' => 'required|array',
            'tpl_id' => 'required|integer',
            'area_type' => ['required', Rule::in(0,1,2,3,4,5)],
        ]);
        if ($validator->fails()) {
            return $this->response->error($validator->errors()->first(), 500);
        }

        return  $this->cmsContentSetTemplate->subjectTwo($request->all());

    }


    //内容发布
    public function release(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'tpl_id' => 'required|integer',
        ]);
        if ($validator->fails()) {
            return $this->response->error($validator->errors()->first(), 500);
        }

        return  $this->cmsContentSetTemplate->release($request->all());

    }


    //内容预览
    public function preview(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'tpl_id' => 'required|integer',
            'type' => 'required|integer',
        ]);
        if ($validator->fails()) {
            return $this->response->error($validator->errors()->first(), 500);
        }

        return  $this->cmsContentSetTemplate->preview($request->all());

    }

    //v0.3兑换商城内容预览
    public function exchangeMall(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'tpl_id' => 'required|integer',
        ]);
        if ($validator->fails()) {
            return $this->response->error($validator->errors()->first(), 500);
        }

        return  $this->cmsContentSetTemplate->exchangeMall($request->all());

    }

    //根据城市ID获取首页模板名称
    public function templateName(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'city_id' => 'required|integer'
        ]);
        if ($validator->fails()) {
            return $this->response->error($validator->errors()->first(), 500);
        }
        return $this->cmsContentSetTemplate->getTemplate($request->only('city_id'));

    }

    //删除某模板内容
    public function templateSetDelete(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'id' => 'required|exists:cms_content_template_set'
        ]);
        if ($validator->fails()) {
            return $this->response->error($validator->errors()->first(), 500);
        }
        return $this->cmsContentSetTemplate->templateSetDelete($request->only('id'));

    }


}