<?php
namespace App\Http\Controllers\V2;
use App\Http\Controllers\BaseController;
use App\Repositories\V2\FloorRepository;
use App\Transformers\FloorTransformer;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use League\Fractal\Resource\Collection;
use League\Fractal\Manager;
use Illuminate\Validation\Rule;
use League\Fractal\Pagination\IlluminatePaginatorAdapter;
/**
 * Created by PhpStorm.
 * User: qinyaer
 * Date: 2019/6/5
 * Time: 上午10:51
 */

class FloorController extends BaseController
{
    public function __construct(FloorRepository $floorRepository)
    {
        $this->floorRepository = $floorRepository;
    }

    //楼层列表
    public function index(Request $request)
    {
        $floor = $this->floorRepository->index($request->all());

        $fractal = new Manager();
        $resource = new Collection($floor, new FloorTransformer());
        $resource->setPaginator(new IlluminatePaginatorAdapter($floor));
        $data = $fractal->createData($resource)->toArray();
        $data['extra'] = [
            'filters' => [
            ],
            'columns' => [
                'id',
                'name',
                'floor_location',
                'group_ids',
                'floor_type',
                'is_open',
                'updated_at',
            ]
        ];
        return $data;
    }


    //新建楼层
    public function create(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'name' => 'required|string',
            'floor_location' => 'required|integer',
        ]);

        if ($validator->fails()) {
            return $this->response->error($validator->errors()->first(), 500);
        }
        return  $this->floorRepository->create($request->all());

    }


    //编辑楼层
    public function edit(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'id' => 'required|exists:cms_floor',
            'name' => 'required|string',
            'floor_location' => 'required|integer',
            'group_ids' => 'required',
            'floor_type' => 'required|integer',
            'is_open' => ['required', Rule::in(0,1)],
        ]);

        if ($validator->fails()) {
            return $this->response->error($validator->errors()->first(), 500);
        }
        return  $this->floorRepository->edit($request->all());
    }


    //修改楼层状态
    public function editStatus(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'id' => 'required|exists:cms_floor',
            'is_open' => ['required', Rule::in(0,1)],
        ]);

        if ($validator->fails()) {
            return $this->response->error($validator->errors()->first(), 500);
        }
        return  $this->floorRepository->editStatus($request->all());
    }

    //关闭banner修改楼层状态
    public function floorBind(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'banner_id' => 'required|integer',
        ]);

        if ($validator->fails()) {
            return $this->response->error($validator->errors()->first(), 500);
        }
        return  $this->floorRepository->floorBind($request->all());
    }
}