<?php

/*
|--------------------------------------------------------------------------
| Application Routes
|--------------------------------------------------------------------------
|
| Here is where you can register all of the routes for an application.
| It is a breeze. Simply tell Lumen the URIs it should respond to
| and give it the Closure to call when that URI is requested.
|
*/

$api = app('Dingo\Api\Routing\Router');

$api->version('v1', [
    'namespace' => 'App\Http\Controllers',
], function ($api) {
    //登录

    $api->post('login', 'AuthController@authenticate');


    $api->group(['middleware' => 'auth:api'], function ($api) {
        //注册
        $api->post('reg', 'AuthController@register');
        //登出
        $api->post('logout', 'AuthController@logout');
        //刷新身份令牌
        $api->post('refresh', 'AuthController@refresh');
    });

    $api->group(['middleware' => 'jwt.chxq_auth'], function ($api) {
        //用户列表

        $api->get('user', 'UserController@index');
        //用户列表
        $api->get('memberList', 'MemberController@memberList');
        //用户详情
        $api->post('memberView', 'MemberController@view');
        //修改状态
        $api->put('updateStatus', 'MemberController@updateStatus');
        //设置属性
        $api->post('setAttr', 'MemberController@setAttr');
        //公共配置
        $api->post('configIndex', 'ConfigController@index');


        //专题列表
        $api->get('/subject/index', 'CmsSubjectController@index');
        //新增专题
        $api->post('/subject/create', 'CmsSubjectController@create');
        //查看某专题
        $api->get('/subject/view', 'CmsSubjectController@view');
        //编辑专题
        $api->put('/subject/edit', 'CmsSubjectController@edit');
        //删除专题
        $api->delete('/subject/delete', 'CmsSubjectController@delete');
        //列表修改专题状态
        $api->post('/subject/editStatus', 'CmsSubjectController@editStatus');


        //模板列表
        $api->get('/template/index', 'CmsContentTemplateController@index');
        //新建模板
        $api->post('/template/create', 'CmsContentTemplateController@create');
        //列表修改模板状态
        $api->put('/template/edit', 'CmsContentTemplateController@edit');
        //列表修改模板名称
        $api->put('/template/editName', 'CmsContentTemplateController@editTemplateName');

        //banner设置
        $api->post('/templateSet/bannerSet', 'CmsContentTemplateSetController@bannerSet');
        //专题广告设置
        $api->post('/templateSet/advertisement', 'CmsContentTemplateSetController@advertisement');
        //商品楼层设置
        $api->post('/templateSet/floorSet', 'CmsContentTemplateSetController@floorSet');
        //分类专题设置
        $api->post('/templateSet/categorySet', 'CmsContentTemplateSetController@categorySet');
        //左一右二配置
        $api->post('/templateSet/subjectOne', 'CmsContentTemplateSetController@subjectOne');
        //上一下三配置
        $api->post('/templateSet/subjectTwo', 'CmsContentTemplateSetController@subjectTwo');

        //模板内容删除
        $api->delete('/templateSetDelete', 'CmsContentTemplateSetController@templateSetDelete');

        //内容发布
        $api->post('/templateSet/release', 'CmsContentTemplateSetController@release');
        //内容预览
        $api->post('/templateSet/preview', 'CmsContentTemplateSetController@preview');
        //点击内容配置
        $api->post('/templateSet/set', 'CmsContentTemplateSetController@set');

        //获取对应城市模板名称
        $api->get('templateName', 'CmsContentTemplateSetController@templateName');

        //v0.3兑换商城内容预览
        $api->post('/templateSet/exchangeMall', 'CmsContentTemplateSetController@exchangeMall');
    });
});

$api->version('v1', [
    'namespace' => 'App\Http\Controllers\V2',
], function ($api) {

    $api->group([
        'prefix' => 'v2'
    ], function ($api) {
        $api->group(['middleware' => 'jwt.chxq_auth'], function ($api) {
            //banner专题计数
            $api->post('countSubject', 'CmsContentTemplateSetController@countSubject');

            //楼层列表
            $api->get('/floor/index', 'FloorController@index');
            //新增楼层
            $api->post('/floor/create', 'FloorController@create');
            //编辑楼层
            $api->put('/floor/edit', 'FloorController@edit');
            //修改楼层状态
            $api->put('/floor/editStatus', 'FloorController@editStatus');
            //关闭banner修改楼层状态
            $api->post('floorBind', 'FloorController@floorBind');
            //新增推荐话题
            $api->post('topicGroup', 'TopicGroupController@create');
            //推荐话题组列表
            $api->get('topicGroup', 'TopicGroupController@index');
            //推荐话题组编辑
            $api->put('topicGroup', 'TopicGroupController@edit');
            //详情
            $api->get('topicGroup/view', 'TopicGroupController@view');
            //删除
            $api->delete('topicGroup', 'TopicGroupController@delete');
            //新增视频组
            $api->post('videoGroup', 'VideoGroupController@create');
            //视频组列表
            $api->get('videoGroup', 'VideoGroupController@index');
            //视频组详情
            $api->get('videoGroup/view', 'VideoGroupController@view');
            //修改
            $api->put('videoGroup', 'VideoGroupController@edit');
            //新增视频组内容
            $api->post('videoGroup/AddVideoInfo', 'VideoGroupController@addVideoInfo');
            //删除内容
            $api->delete('videoGroup/DeleteVideoInfo', 'VideoGroupController@deleteVideoInfo');
            //删除分组
            $api->delete('videoGroup', 'VideoGroupController@delete');
            //推荐用户列表
            $api->get('memberGroup', 'MemberGroupController@index');
            //推荐用户详情
            $api->get('memberGroup/view', 'MemberGroupController@view');
            //推荐用户添加
            $api->post('memberGroup', 'MemberGroupController@create');
            //推荐用户修改
            $api->put('memberGroup', 'MemberGroupController@edit');
            //推荐用户删除
            $api->delete('memberGroup', 'MemberGroupController@delete');
            //删除用户组内容
            $api->delete('memberGroup/DeleteMemberInfo', 'MemberGroupController@deleteMemberInfo');
            //设置推荐
            $api->put('memberGroup/SetIsSuggest', 'MemberGroupController@setIsSuggest');
            //设置内容排序
            $api->put('memberGroup/SetInfoSort', 'MemberGroupController@setSort');

        });

    });
});